/* $Id: clc_assert.h,v 1.9 2003/06/01 16:01:27 hfuru Exp $ */

#ifdef __cplusplus
extern "C" {
#endif

#ifndef CLC_ASSERT_H	/* This only guards the top half of the file. */
#define CLC_ASSERT_H	/* The bottom half is processed each time.    */

#include <stddef.h>

/**
 * Copyright (c) 2003, B. Augestad (boa@metasystems.no)
 *
 * Replacement/improvement of the assert macro.
 * This file defines three macros.
 * - clc_assert()          - Replaces assert()
 * - clc_assert_not_null() - Use it to test function pointer arguments
 * - clc_assert_arg()      - Use it to test other function arguments.
 *
 * We use these macros within libclc to help the users of libclc
 * to track down their errors. We also provide clc_assert to our users,
 * so that they can use clc_assert macros in their own code.
 *
 * NOTE: The documentation will be moved to a separate document later,
 * just waiting for the doc. guidelines. The documentation will be
 * split into several sections, where policies will be a part of
 * the "Advanced Topics" section. 
 *
 * NOTE 2: CLC_DEBUG is a replacement for NDEBUG, but with 
 * inverted meaning. The default is that CLC_DEBUG is undefined.
 */

/**
 * Output policies are used to direct the output 
 * if an assertion fails. clc_assert support multiple
 * output destinations, which means that you can print
 * to both stdout and stderr at the same time. This is
 * good if you redirect either stdout or stderr to a 
 * file.
 *
 * You can also turn output off completely. 
 *
 * Output policies can be changed while the program runs,
 * as many times as you prefer. The current implementation
 * is process wide. Thread specific policies can be implemented.
 */
#define CLC_ASSERT_OUTPUT_STDERR	0x01
#define CLC_ASSERT_OUTPUT_STDOUT	0x02
#define CLC_ASSERT_OUTPUT_NOTHING	0x04
#define CLC_ASSERT_OUTPUT_FILE		0x08

/**
 * Set the current output policy.
 * The file argument is required if policy includes
 * CLC_ASSERT_OUTPUT_FILE, NULL can be used if file
 * isn't required.
 *
 * The file argument will be stored as a char pointer,
 * the contents will not be copied. This is to avoid unneeded
 * overhead, but implies that the memory pointed to must be
 * valid if an error occurs. 
 */
void clc_assert_set_output_policy(int clc_policy, const char* clc_file);

/**
 * Advanced.
 * Three alternatives on what to do if an assertion fails.
 * The program can either call abort(), call exit()
 * or continue to run. 
 * 
 * The default policy is to call abort.
 *
 * exit() will be called with EXIT_FAILURE. Functions
 * registred to run at exit will be executed if halt policy
 * is set to CLC_ASSERT_HALT_EXIT, but not if the policy
 * is CLC_ASSERT_HALT_ABORT.
 */
#define CLC_ASSERT_HALT_NO			0x01
#define CLC_ASSERT_HALT_ABORT		0x02
#define CLC_ASSERT_HALT_EXIT		0x04
void clc_assert_set_halt_policy(int clc_policy);

/**
 * The function prints messages if an assertion fail.
 * It works like printf reg. format strings and arguments. 
 * This function honors the output policy. 
 *
 * If this function prints to a file, then the file will
 * be opened and closed each time. Failure to open or close
 * the file are ignored, and no message is printed if an error occures.
 *
 * Note that the function is also very usable as a trace
 * function, if one ever is needed.
 */
void clc_assert_print(const char *clc_format, ...);

/**
 * Halts the program. clc_assert_halt must adhere to the 
 * current halt policy. 
 */
void clc_assert_halt(void);

#endif /* CLC_ASSERT_H */


/* This part of the file is processed each time the file is #included */

#undef clc_assert
#undef clc_assert_not_null
#undef clc_assert_arg

#ifndef CLC_DEBUG

#define clc_assert(fn, cond)       ((void)0)
#define clc_assert_not_null(fn, a) ((void)0)
#define clc_assert_arg(fn, cond)   ((void)0)

#else

extern const char clc_assert_format[];
extern const char clc_assert_not_null_format[];
extern const char clc_assert_arg_format[];

#define clc_assert(fn, cond)\
	((cond) ? (void)0 :\
		(clc_assert_print(\
			clc_assert_format,\
			#fn,\
			__FILE__,\
			(unsigned long)__LINE__,\
			#cond),\
		 clc_assert_halt()))

#define clc_assert_not_null(fn, param)\
	((param) != NULL ? (void)0 :\
		(clc_assert_print(\
			clc_assert_not_null_format,\
			#fn,\
			__FILE__,\
			(unsigned long)__LINE__,\
			#param),\
		 clc_assert_halt()))

#define clc_assert_arg(fn, cond)\
	((cond) ? (void)0 :\
		(clc_assert_print(\
			clc_assert_arg_format,\
			#fn,\
			__FILE__,\
			(unsigned long)__LINE__,\
			#cond),\
		 clc_assert_halt()))

#endif /* CLC_DEBUG */

#ifdef __cplusplus
}
#endif
